
import sys, pprint
from pymongo import MongoClient
import constrains_spidy.contrain_checker as constrains

"""
Dependencies of the Library: 

pip3.5 install pymongo 

Metadata of Library 

Mandatory 
---------
mongo_host = DB hostname 
mongo_port = DB Port 
mongo_user = Username for DB access 
mongo_user_pass = Password for DB access 
mongo_db_name = Mongo Collection Name 
mongo_uri = URI format of mongo 

Optional 
---------
mongo_collection = DB Collection Name. 

"""
class mongo_functions:

      def __init__(self,baseline={}):
          self.__baseline__ = baseline

          #### Constrain Checks
          metadata_headers = ["mongo_host", "mongo_user","mongo_user_pass", \
                             "mongo_port", "mongo_db_name" ]
          constrain_check = constrains.constrain_check()
          constrain_check.metadata_checks(self.__baseline__, metadata_headers)

          ###### Connectivity checks
          constrain_check.check_network(self.__baseline__['mongo_host'],self.__baseline__['mongo_port'])

          ###### mongo Connectivity Code.
          try:
                if  self.__baseline__.__contains__("mongo_uri"):
                    self.__client = MongoClient(self.__baseline__['mongo_uri'])
                else:
                    self.__client = MongoClient(self.__baseline__['mongo_host'],int(self.__baseline__['mongo_port']))
          except TypeError or ConnectionAbortedError or ConnectionResetError or ValueError or ConnectionError :
                 print("Connectivity Issues with MongoDB, Kindly check and validated the DB connectivity settings Manually")
                 sys.exit(1)

          try:
                self.__db = self.__client[self.__baseline__['mongo_db_name']]
          except ValueError:
                print ("Problems with DB name or DB name does not Exists Check with MongoDB Administrator")


      def mongo_writes(self):
          pass

      def mongo_get_info(self):
          ### Gather Database information
          list_data = {}
          list_data["collections"] = self.__db.collection_names(include_system_collections=True)
          list_data["profiling_info"] = self.__db.profiling_info()
          list_data["last_status"] = self.__db.last_status()
          list_data["sizeof"] = self.__db.__sizeof__()
          list_data["connectivity_details"] = self.__db.__str__()

          return list_data

      def mongo_get_db(self):
          ### Dump all DB Collections.
          list_data = {}
          list_documents = self.__db.collection_names()
          for docs in list_documents:
              list_data[docs] = []
              cursor  = self.__db.get_collection(name=docs).find({})
              for items in cursor:
                  list_data[docs].append(items)
              cursor.close()
          return list_data

      def mongo_get_collection(self,collection_name,search_filter=None):

          ### Dump given Collection Documents.
          list_data = []
          if search_filter == None:
             cursor = self.__db.get_collection(name=collection_name).find({},{'_id': False})
          else:
             cursor = self.__db.get_collection(name=collection_name).find(search_filter)

          for items in cursor:
              list_data.append(items)

          cursor.close()
          return list_data

      def mongo_new_collection(self,collection_name,data):

          ### Data Quality Check
          constrain_check = constrains.constrain_check()
          constrain_check.mongo_collection_creation_constrain(data)

          ### Check if Collection Exists
          check_collection  = self.mongo_get_info()
          if check_collection["collections"].__contains__(collection_name):
             print("Collection already Exists, Kindly use the mongo_append_collection function")
             sys.exit(1)

          ### Create new collection.
          self.__db.create_collection(name=collection_name).insert_many(data)
          return "Success"

      def mongo_append_collection(self,collection_name,data):

          ### Data Quality Check
          constrain_check = constrains.constrain_check()
          constrain_check.mongo_collection_creation_constrain(data)

          ### Check if Collection Exists
          """
          check_collection  = self.mongo_get_info()
          if not check_collection["collections"].__contains__(collection_name):
             print("Collection Does not Exists, Kindly use the mongo_new_collection function")
             sys.exit(1)
          """

          ### Append to Collection
          self.__db.get_collection(collection_name).insert(data)
          return "Success"

      def mongo_remove_collection(self,collection_name):

          ### Remove collection
          self.__db.drop_collection(collection_name)



from distutils.core import setup
setup(
  name = 'ansible_tools_spidy',
  packages = ['FILE_spidy','Utilities_spidy','constrains_spidy'], # this must be the same as the name above
  version = '0.11',
  description = 'ansible_tools_spidy allows you to store and retreive playbooks in mongodb using passwordless auth, Security Wrappers in the next Release command :- /usr/local/bin/ansible_playbook & yaml_to_mongo',
  author = 'Anish V Tharagar',
  author_email = 'anish.tharagar@gmail.com',
  url = 'https://github.com/anishtharagar/code_repo_public.git', # use the URL to the github repo
  download_url = 'https://github.com/anishtharagar/code_repo_public/packaging/', # I'll explain this in a second
  keywords = ['ansible', 'playbooks', 'automation'], # arbitrary keywords
  install_requires = [ 'pyaml','pymongo','libmagic', 'termcolor' ],
  classifiers = [],
  data_files=[('/usr/local/bin',['Tools_spidy/ansible_playbook','Tools_spidy/yaml_to_mongo']),
	      ('/usr/share',['files_repo/mongo_connect.json'])],
  license = 'MIT',
)



import json
import sys
import yaml
import pprint
from termcolor import colored
import constrains_spidy.contrain_checker as constrains

sys.path.append("../")

"""
Dependencies of the Library: 

git clone https://github.com/ahupp/python-magic
pip3.5 install libmagic 
brew install libmagic 
pip3.5 termcolor 


Metadata of Library 

file_name = self explanatory 
file_path = self explanatory 

"""

class file_functions:

       def __init__(self,baseline={}):
           self.__baseline__ = baseline

           ### Metadata Constrain Checks
           metadata_headers = ["file_name", "file_loc"]
           constrain_check = constrains.constrain_check()
           constrain_check.metadata_checks(self.__baseline__,metadata_headers)

           ### File Path Exists Constrain Check
           constrain_check.check_file_exists(self.__baseline__["file_loc"])

##############################    JSON File Handling ######################################

       def get_json(self):

           ### File Exists
           constrain_check = constrains.constrain_check()
           constrain_check.check_file_exists(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"])

           ### File Type Check
           constrain_check.check_file_type(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"],"text/plain")

           ### Read from json file and return a dictionary
           try:
                with open(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"], 'r') as json_file:
                     json_string = json.load(json_file)
                return json_string

           except ValueError:
                 print(colored("JSON Decoding Problem Check " + self.__baseline__["file_name"] + " for Formatting Issues",'blue',attrs=['reverse', 'blink']))
                 print()
                 sys.exit(1)

       def write_json(self,json_obj=None):

           #### Check if the Json Object is passed as parameter
           if(json_obj == None):
              print(colored("Kindly pass the Json Object as Parameter to the Function",'blue',attrs=['reverse', 'blink']))
              sys.exit(1)

           try:
                with open(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"], 'w') as json_file:
                     json.dump(json_obj,json_file)

           except PermissionError:
               print(colored("Kindly Fix Permissions issue for " + self.__baseline__["file_loc"] + " before proceeding" , 'blue',attrs=['reverse', 'blink']))
               sys.exit(1)

#######################################  Yaml File Handling ##################################

       def get_yaml(self):

           ### File Exists
           constrain_check = constrains.constrain_check()
           constrain_check.check_file_exists(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"])

           ### File Type
           constrain_check.check_file_type(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"],"text/plain")

           ### Read from Yaml file and return Dict
           try:
                with open(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"], 'r') as yaml_file:
                     yaml_string = yaml.load(yaml_file)

                return yaml_string

           except ValueError:
                 print(colored("YAML Decoding Problem Check " + self.__baseline__["file_name"] + " for Formatting Issues",'blue',attrs=['reverse', 'blink']))
                 print()
                 sys.exit(1)

       def write_yaml(self,yaml_obj=None):

           #### Check if the Json Object is passed as parameter
           if(yaml_obj == None):
              print(colored("Kindly pass the Yaml Object as Parameter to the Function",'blue',attrs=['reverse', 'blink']))
              sys.exit(1)

           try:
                with open(self.__baseline__["file_loc"] + "/" + self.__baseline__["file_name"], 'w') as yaml_file:
                     yaml.dump(yaml_obj,yaml_file)

           except PermissionError:
               print(colored("Kindly Fix Permissions issue for " + self.__baseline__["file_loc"] + " before proceeding" , 'blue',attrs=['reverse', 'blink']))
               sys.exit(1)

       def file_encrypt(self,public_key):
           pass
       def file_decrypt(self,private_key):
           pass
